module IndexExample
where

import Data.Char
import Data.List

import           Data.Set(Set)
import qualified Data.Set as S

import Index

import Zitate

-- ------------------------------------------------------------
--
-- der Index-Aufbau

zitateIx	:: Index
zitateIx	= foldr (indexDocument' istVonInteresse) emptyIndex
		  .
		  map (\ (n, z) -> (show n, scanText z) )
		  $
		  zitate

-- ------------------------------------------------------------
--
-- der Filter fuer die unwichtigen Woerter

istVonInteresse w
    = length w > 1
      &&
      w `notElem` [ "und", "oder", "nicht",
		    "ich", "du", "er", "sie", "es", "wir", "ihr",
		    "bin", "bist", "ist", "sind", "seid",
		    "der", "die", "das", "dem", "den", "des",
		    "ein", "eine", "einer",
		    "nur",
		    "mein", "meine", "meiner", "dein", "sein",
		    "ihn", "ihr", "uns",
		    "von", "vom",
		    "wer", "wie", "was",
		    "wieso", "weshalb", "warum",
		    "wenn", "werde", "werden",
		    "wurde", "wurden", "will",
		    "zum", "zur"
		  ]
      &&
      not (all isDigit w)

-- ------------------------------------------------------------
--
-- die Anfragen

wortSuche w
    = gefundeneZitate (`S.member` res)
    where
    res = documentNames . searchIndex w $ zitateIx

praefixSuche p
    = gefundeneZitate (`S.member` res)
    where
    res = documentNames . prefixSearchIndex p $ zitateIx

-- ------------------------------------------------------------

gefundeneZitate	:: (DocumentName -> Bool) -> IO()
gefundeneZitate isRes
    = putStrLn
      $
      concatMap (\ (_wer, was) -> breakLines was ++ "\n")
      .
      sort
      .
      filter (\ (wer, _was) -> isRes (show wer))
      $
      zitate

alleZitate	:: IO()
alleZitate	= gefundeneZitate (const True)

-- ------------------------------------------------------------
--
-- wichtige einfache Anfragen

bayern	= wortSuche "Bayern"
merkel  = wortSuche "Merkel"
willi	= wortSuche "Willi"
loddar	= praefixSuche "Matt"
trainer	= praefixSuche "Train"

-- ------------------------------------------------------------
--
--

breakLines s
    = line ++ word ++ "\n"
      ++
      ( if null rest2
	then ""
	else "\t" ++ breakLines (dropWhile isSpace rest2)
      )
    where
    (line, rest) = splitAt 50 s
    (word, rest2) = span (not . isSpace) rest

-- ------------------------------------------------------------
