module ParsedQuery(query, runLex, parseQuery)
where

import Query
import IndexExample2(suche)

import IO

import Text.ParserCombinators.Parsec
import Text.ParserCombinators.Parsec.Expr
import Text.ParserCombinators.Parsec.Language 
import qualified Text.ParserCombinators.Parsec.Token as P

-- ------------------------------------------------------------
--
-- token parser

qlex	= P.makeTokenParser wordDef
    where
    charDef = oneOf ( ['a'..'z'] ++ ['A'..'Z'] ++ "\196\214\220\228\246\252\223" )
    wordDef = emptyDef { identStart = charDef, identLetter = charDef }

qword		= P.identifier qlex
qop		= P.reservedOp qlex

whiteSpace	= P.whiteSpace qlex
parens    	= P.parens qlex
symbol    	= P.symbol qlex
reservedOp	= P.reservedOp qlex
lexeme		= P.lexeme qlex

-- ------------------------------------------------------------
--
-- query parser

query	:: Parser Query
query	= buildExpressionParser qtable qfactor
	  <?> "query"

qtable
    = [ [ op '>' (\ os x y -> QBin (QFollow  (length os + 1)) x y) AssocRight
	, op '+' (\ os x y -> QBin (QContext (length os + 1)) x y) AssocRight
	]
      , [ op '&' (const (QBin QAnd)) AssocLeft ]
      , [ op '|' (const (QBin QOr))  AssocLeft ]
      ]
    where
    op c f assoc
	= Infix ( do
		  os <- lexeme (many1 (char c))
		  return (f os)
		) assoc

qfactor	:: Parser Query
qfactor
    = do
      ql <- many1 qsimple
      return ( foldr1 (QBin QPhrase) ql )

qsimple	:: Parser Query
qsimple
    = parens query
      <|>
      qprim
      <?> "simple query"

qprim	:: Parser Query
qprim
    = do
      w <- qword
      option (QWord w) ( do
			 lexeme (many1 (char '.'))
			 return (QPrefix w)
		       )


-----------------------------------------------------------

run :: Show a => Parser a -> String -> IO ()
run p input
        = case (parse p "" input) of
            Left err -> do{ putStr "parse error at "
                          ; print err
                          }
            Right x  -> print x


runLex :: Show a => Parser a -> String -> IO ()
runLex p 
        = run (do{ whiteSpace 
                 ; x <- p
                 ; eof
                 ; return x
                 }
              ) 

parseQuery	:: String ->  Either ParseError Query
parseQuery
    = parse ( do
	      whiteSpace
	      res <- query
	      eof
	      return res
	    ) ""
