module Query
where

import           Data.Map(Map)
import qualified Data.Map as M
import           Data.Set(Set)
import qualified Data.Set as S

import Index

-- ------------------------------------------------------------
--
-- the query data types

data Query
    = QWord	Word
    | QPrefix	Word
    | QBin      QOp Query Query
      deriving (Show)

data QOp
    = QAnd
    | QOr
    | QPhrase
    | QFollow  Int
    | QContext Int
      deriving (Show)

-- ------------------------------------------------------------
--
-- the query evaluator

selectDocs	:: Query -> Index -> DocumentNames
selectDocs q	= documentNames . select q

select		:: Query -> Index -> Occurences

select (QWord w) ix
    = searchIndex w ix

select (QPrefix p) ix
    = prefixSearchIndex p ix

select (QBin op q1 q2) ix
    = f r1 r2
    where
    f  = op2func op
    r1 = select q1 ix
    r2 = select q2 ix

-- ------------------------------------------------------------
--
-- operator -> result set combinator

op2func QAnd
    = intersectOcc

op2func QOr
    = unionOcc

op2func QPhrase
    = followedByOcc

op2func (QFollow n)
    = \ r1 r2 -> r1 `S.intersection` (n `nearByOcc` r2)

op2func (QContext n)
    = \ r1 r2 ->
      ( ( r1 `S.intersection` (n `nearByOcc` r2) )
	`S.union`
	( r2 `S.intersection` (n `nearByOcc` r1) )
      ) 

-- ------------------------------------------------------------
--
-- result set combination

unionOcc	:: Occurences -> Occurences -> Occurences
intersectOcc	:: Occurences -> Occurences -> Occurences
followedByOcc	:: Occurences -> Occurences -> Occurences
shiftOcc	:: Int        -> Occurences -> Occurences
nearByOcc	:: Int        -> Occurences -> Occurences

occ1 `unionOcc` occ2
    = occ1 `S.union` occ2

occ1 `intersectOcc` occ2
    = S.filter ((`S.member` docs) . fst) occ1
      `S.union`
      S.filter ((`S.member` docs) . fst) occ2
    where
    docs = documentNames occ1
	   `S.intersection`
	   documentNames occ2

i `shiftOcc` occ
    = S.fold (\ (n, p) s -> S.insert (n, p - i) s) S.empty occ

occ1 `followedByOcc` occ2
    = occ1 `S.intersection` (1 `shiftOcc` occ2)

i `nearByOcc` occ
    = S.unions . map (\ i -> i `shiftOcc` occ) $ [1..i]
